﻿using Kivii.Finances.Entities;
using Kivii.Linq;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Transforms
{
    /// <summary>
    /// 此接口用于以下情况
    /// 1.到账充值到客户存款账户
    /// 2.退款存入退款账户
    /// 3.非业务收入存入个人现金,刷卡等账户
    /// </summary>
    [Api(Description = "收款转账")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class PaymentTransfer : RestfulExecution<Payment>
    {
        public Guid AccountKvid { get; set; }

        /// <summary>
        /// 类型:Split,UnBiz,Refund,Cash,Pos,AliPay,Wechat
        /// </summary>
        public Guid PaymentKvid { get; set; }

        public decimal Amount { get; set; }

        public string Remark { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            if (AccountKvid == Guid.Empty || PaymentKvid == Guid.Empty) throw new Exception("请选择要处理的到账信息!");
            if (Amount <= 0) throw new Exception("入账金额不能小于等0!");

            var conn = KiviiContext.GetOpenedDbConnection<Account>();
            var payment = conn.SingleById<Payment>(PaymentKvid);
            if (payment == null) throw new Exception("不存在的收款信息!");
            if (payment.OffsetKvid != Guid.Empty) throw new Exception("此收款已冲账!");
            if (payment.Type != PaymentType.UnBiz && payment.Type != PaymentType.Refund && payment.Type != PaymentType.Split && payment.Type != PaymentType.Assign && payment.Type != PaymentType.Cash && payment.Type != PaymentType.AliPay && payment.Type != PaymentType.WeChat && payment.Type != PaymentType.Pos) throw new Exception("不支持收款充值操作类型!");
            if (payment.Amount - payment.AmountUsed < Amount) throw new Exception($"此到账可使用金额不足,仅剩:{payment.Amount - payment.AmountUsed}元");

            var accountPayee = conn.SingleById<Account>(AccountKvid);
            if (accountPayee == null) throw new Exception("不存在此账户");
            if (accountPayee.Type != AccountType.Cash && accountPayee.Type != AccountType.Pos && accountPayee.Type != AccountType.Deposit && accountPayee.Type != AccountType.Refund) throw new Exception("不支持的账户类型");

            //var accountPayer = accountPayee.Currency.GetBalanceAccount();
            //accountPayer.ThrowIfNull($"未能正确获取系统{accountPayee.Currency}平衡账户!请联系管理员!");

            //生成转账Payment
            Payment transfer = null;
            //生成账户明细
            List<AccountDetail> accountDetails = new List<AccountDetail>();
            List<Pay> pays = new List<Pay>();

            //3
            if (payment.Type == PaymentType.UnBiz)
            {
                if (accountPayee.Type != AccountType.Cash && accountPayee.Type != AccountType.Pos) throw new Exception("非业务收款,仅支持现金或刷卡个人账户充值!");
                transfer = payment.Transfer(accountPayee, Amount, $"非业务入账{(Remark.IsNullOrEmpty() ? string.Empty : ":" + Remark)}", out Pay pay, out AccountDetail accountDetail);
                accountDetails.Add(accountDetail);
                pays.Add(pay);
            }
            //2
            else if (payment.Type == PaymentType.Refund)
            {
                if (accountPayee.Type != AccountType.Refund) throw new Exception("退款收款,仅支持退款账户充值!");
                transfer = payment.Transfer(accountPayee, Amount, $"退款入账{(Remark.IsNullOrEmpty() ? string.Empty : ":" + Remark)}", out Pay pay, out AccountDetail accountDetail);
                accountDetails.Add(accountDetail);
                pays.Add(pay);
            }
            //1
            else
            {
                if (accountPayee.Type != AccountType.Deposit) throw new Exception("业务收款,仅支持客户存款账户充值!");
                transfer = payment.Transfer(accountPayee, Amount, $"到账充值{(Remark.IsNullOrEmpty() ? string.Empty : ":" + Remark)}", out Pay pay, out AccountDetail accountDetail);
                accountDetails.Add(accountDetail);
                pays.Add(pay);
            }
            var rtns = new RestfulExecutionResponse<Payment>();
            rtns.Results = new List<Payment>();

            var trans = conn.OpenTransaction();
            try
            {
                transfer.SerialNumber = transfer.GetSubSerialNumber(conn);
                conn.Insert(transfer);
                rtns.Results.Add(transfer);
                accountDetails.ForEach(o => o.Insert(conn));
                payment.RecalculateAmountUsed(conn);//重新计算AmountUsed!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
                if (payment.Type == PaymentType.Split) payment.RecalculateAssignAmount(conn);
                //if (parentPayment != null) conn.UpdateOnly(parentPayment);
                foreach (var item in pays)
                {
                    conn.Insert(item);
                }
                trans.Commit();
            }
            catch (Exception ex)
            {
                trans.Rollback();
                throw ex;
            }

            return rtns;
        }
    }

    //(重点测试下2021.09.14)
    [Api(Description = "收款转账撤销")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class PaymentTransferCancel : RestfulExecution<Payment>
    {
        /// <summary>
        /// 类型:Transfer
        /// </summary>
        public List<Guid> PaymentKvids { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            PaymentKvids.ThrowIfNullOrEmpty("请传入要撤销转账的内容!");
            var conn = KiviiContext.GetOpenedDbConnection<Payment>();
            var payments = conn.SelectByIds<Payment>(PaymentKvids);
            payments.ThrowIfNullOrEmpty("未找到要撤销转账的信息!");
            (payments.Count != PaymentKvids.Count).ThrowIfTrue("所选到账与查询到数据数量不符合!");
            payments.Exists(o => o.Type != PaymentType.Transfer).ThrowIfTrue("存在不正确的到账类型!");
            payments.Exists(o => o.OffsetKvid != Guid.Empty).ThrowIfTrue("存在已撤销的转账!");
            payments.Exists(o => o.ParentKvid == Guid.Empty).ThrowIfTrue("存在不完整的转账数据!");

            var parentPayments = conn.SelectByIds<Payment>(payments.ConvertAll(o => o.ParentKvid));
            parentPayments.ThrowIfNullOrEmpty("未找到夫级付款,数据错误!");
            //var rootPayments = conn.SelectByIds<Payment>(parentPayments.ConvertAll(o => o.ParentKvid));
            var accountPayees = conn.SelectByIds<Account>(payments.ConvertAll(o => o.PayeeAccountKvid));
            accountPayees.ThrowIfNullOrEmpty("未找到收款账户!");
            var allPays = conn.Select<Pay>(o => o.FromType == PayType.Payment && Sql.In(o.FromKvid, payments.ConvertAll(p => p.Kvid)));

            var offsets = new List<Payment>();//新生成的Offset数据
            var orginOffsets = new List<Payment>();//原Transfer数据更新Offset
            var payOffsets = new List<Pay>();//新生成Offset的Pay
            var orginPayOffsets = new List<Pay>();//原Pay作废
            var totalAccountDetails = new List<AccountDetail>();//生成的账户明细
            
            foreach (var item in payments)
            {
                //var parent = parentPayments.FirstOrDefault(o => o.Kvid == item.ParentKvid);
                //parent.ThrowIfNull($"未找到{item.SerialNumber}当前的到账上级节点!");
                var accountPayee = accountPayees.FirstOrDefault(o => o.Kvid == item.PayeeAccountKvid);
                accountPayee.ThrowIfNull($"未找到{item.SerialNumber}对应收款账户!");
                (accountPayee.Amount < item.Amount).ThrowIfTrue("账户余额不足无法撤销已充值金额!");
                #region 新增流水,原收款账户新增扣款明细
                var payeeAaccountDetail = new AccountDetail();
                payeeAaccountDetail.AccountKvid = accountPayee.Kvid;
                payeeAaccountDetail.BizId = item.SerialNumber;
                payeeAaccountDetail.BizKvid = item.Kvid;
                payeeAaccountDetail.BizType = typeof(Payment).FullName;

                payeeAaccountDetail.PayerKvid = item.PayeeKvid;
                payeeAaccountDetail.PayerName = item.PayeeName;
                payeeAaccountDetail.PayerAccountKvid = accountPayee.Kvid;
                payeeAaccountDetail.PayerAccountName = accountPayee.Name;
                payeeAaccountDetail.PayerAccountSerialNumber = accountPayee.SerialNumber;

                payeeAaccountDetail.PayeeKvid = item.PayerKvid;
                payeeAaccountDetail.PayeeName = item.PayerName;
                //payeeAaccountDetail.PayeeAccountKvid = accountPayer.Kvid;
                //payeeAaccountDetail.PayeeAccountName = accountPayer.Name;
                //payeeAaccountDetail.PayeeAccountSerialNumber = accountPayer.SerialNumber;

                payeeAaccountDetail.AmountPayment = item.Amount;
                payeeAaccountDetail.Summary = "撤销转账";
                #endregion
                totalAccountDetails.Add(payeeAaccountDetail);

                var offset = item.Offset($"撤销转账");//tranfer的Payment作废
                offsets.Add(offset);
                orginOffsets.Add(item);

                if (!allPays.IsNullOrEmpty())//Transfer对应的Pay也要作废
                {
                    var currentPays = allPays.Where(o => o.FromKvid == item.Kvid).ToList();
                    if (!currentPays.IsNullOrEmpty())
                    {
                        foreach (var pay in currentPays)
                        {
                            var offsetPay = pay.Offset($"撤销转账");
                            payOffsets.Add(offsetPay);
                            orginPayOffsets.Add(pay);
                        }
                    }
                }
            }

            var rtns = new RestfulExecutionResponse<Payment>();
            rtns.Results = new List<Payment>();

            var trans = conn.OpenTransaction();

            try
            {
                foreach (var item in orginOffsets)
                {
                    conn.UpdateOnly(item);
                    rtns.Results.Add(item);
                }

                foreach(var offset in offsets)
                {
                    conn.Insert(offset);
                }

                foreach (var item in orginPayOffsets)
                {
                    conn.UpdateOnly(item);
                }

                foreach (var offset in payOffsets)
                {
                    conn.Insert(offset);
                }

                foreach (var  item in totalAccountDetails)
                {
                    item.Insert(conn);
                }

                foreach( var item in parentPayments)
                {
                    item.RecalculateAmountUsed(conn);
                }

                trans.Commit();
            }
            catch(Exception ex)
            {
                trans.Rollback();
                throw ex;
            }

            return rtns;
        }
    }
}
